import os
import java.io.FileFilter
import re
import tempfile
import shutil

import gatetools
import gate

class FileFilter(java.io.FileFilter):
    def __init__(self,re):
        self.re = re
    def accept(self, f):
        return self.re.match(f.getName()) != None

class Main:
    def __init__(self, cfg):
        self.cfg = cfg

    def getCorpus(self, do_import):
        storepath = self.cfg.storeroot + os.sep + self.cfg.storename
        if do_import:
            os.system("rm -Rf %s" % storepath)
            os.makedirs(storepath)

        corp=gate.Factory.newCorpus(self.cfg.storename)
        if self.cfg.imp_sds:
            if do_import:
                self.sds=gate.Factory.createDataStore(
                    "gate.persist.SerialDataStore", 
                    gatetools.makeURL(storepath));
                corp=self.sds.adopt(corp,None)
            else:
                self.sds=gate.Factory.openDataStore(
                    "gate.persist.SerialDataStore", 
                    gatetools.makeURL(storepath));
                corp=gatetools.createResourceByName(
                    self.sds,
                    "gate.corpora.SerialCorpusImpl", 
                    self.cfg.storename)
        else:
            self.sds = None

        if do_import and self.cfg.imp_dirs:
            for x in self.cfg.imp_dirs:
                if isinstance(x,tuple): p,filter = x
                else: p,filter=x,None
            
                if filter: ff = FileFilter(re.compile(filter))
                else: ff = None

                corp.populate(java.io.File(self.cfg.dataroot + os.sep + p).toURL(),
                              ff, "utf-8", False)

        print 'corpus has %i documents' % corp.size()
        self.corp = corp

    def process(self):
        controller=gate.Factory.createResource("gate.creole.SerialAnalyserController")
        for s in self.cfg.steps: controller.add(s)
        controller.setCorpus(self.corp)
        controller.execute()
        print 'processing finished'

    def foreach(self,h):
        for d in self.corp:
            h(d)


    def learn(self,learningCFG,datasetCFG,learn=False,copylog=None):

        configurl = tempfile.mktemp('.ofai-learning-tmp')
        f = open(configurl,'w')
        f.write("<?xml version=\"1.0\"?>\n<ML-CONFIG>%s %s</ML-CONFIG>" % 
                (learningCFG, datasetCFG))
        f.close()
        
        params=gate.Factory.newFeatureMap()
        params.put("corpus", self.corp)
        corp=gate.Factory.createResource("gate.learning.ShuffledCorpus", params)

        controller=gate.Factory.createResource("gate.creole.SerialAnalyserController")

        import os
        os.system("rm -Rf %s" % (tempfile.gettempdir() + os.sep + "savedFiles/"))

        params=gate.Factory.newFeatureMap()
        params.put("configFileURL", gatetools.makeURL(configurl))
        if learn:
            params.put("learningMode", "EVALUATION")
        else:
            params.put("learningMode", "ProduceFeatureFilesOnly")
        params.put("inputASName", "Original markups")
        params.put("outputASName", "Original markups")
        pr=gate.Factory.createResource("gate.learning.LearningAPIMain", params)
        controller.add(pr)

        controller.setCorpus(corp)
        controller.execute()

        if copylog: shutil.copyfile(tempfile.gettempdir() + os.sep + "savedFiles/logFileForNLPLearning.save", copylog)

    def cleanup(self):
        if self.sds:
            self.corp.sync()
            self.sds.close()
            self.sds = None
            self.corp = None
        os.system('rm -Rf %s' % (tempfile.gettempdir() + os.sep + '*.ofai-learning-tmp'))

gate.Gate.init()
gate.Gate.getCreoleRegister().registerDirectories(java.io.File(gate.Gate.getPluginsHome(), "learning").toURL())
