#!/usr/bin/env jython

from config import BaseConfig
from main import Main
import learning
import process
import gate
import os
#import gate2arff

############################################################################
# processing functions
############################################################################

def tagdoc(d):
    """Process a single GATE document. Set the Rating (i.e. class label) 
    based on the filename (sourceurl). Mark the whole document as 
    'review_text'.
    """
    if '/pos/' in d.getSourceUrl().toString(): classlabel='pos'
    else: classlabel='neg'
    
    fs=gate.util.SimpleFeatureMapImpl()
    fs.put('rating',classlabel)
    d.getAnnotations('Original markups').add(0,d.getContent().size(),'Rating',fs)
    fs=gate.util.SimpleFeatureMapImpl()
    d.getAnnotations('Original markups').add(0,d.getContent().size(),'review_text',fs)

############################################################################
# Configurations
############################################################################
class Cornell700(BaseConfig):
    """Configuration for the Cornell 700/700 Corpus
    """
    dataroot = BaseConfig.dataroot + '/data/cornell700-html'
    imp_dirs = ['neg','pos']

    # set this flag to TRUE if you want to generate a serialised GATE corpus
    imp_sds = False 

    storename = 'cornell700'

    # list of processing steps to apply to the corpus
    steps = [process.sentenceSplitter(),
             process.tokenizer(),
             ]

class Cornell1000(BaseConfig):
    """Configuration for the Cornell 1000/1000 Corpus
    """
    dataroot = BaseConfig.dataroot + '/data/cornell1000-html'
    imp_dirs = ['neg','pos']

    # set this flag to TRUE if you want to generate a serialised GATE corpus
    imp_sds = False

    storename = 'cornell1000'
    # list of processing steps to apply to the corpus
    steps = [process.sentenceSplitter(),
             process.tokenizer(),
             ]

############################################################################
# handle parameters, import corpus, process documents, and 
# generate ARFF files or use the learning API to directly learn a classifier
############################################################################

# instantiate a Cornell700 or Cornell1000 configuration object
import sys
if len(sys.argv) > 1 and sys.argv[1]=='700':
    cfg,prefix = Cornell700(),'cornell700'

elif len(sys.argv) > 1 and sys.argv[1]=='1000':
    cfg,prefix = Cornell1000(),'cornell1000'

else:
    print 'usage: cornell.py [700|1000]'
    sys.exit(-1)

# generate the main object initialized from the configuration
m=Main(cfg)

# retrieve the corpus and perform import of documents
# NOTE: set do_import to False if the imp_sds Flag of the configuration object
# is set to True and documents have already been imported
m.getCorpus(do_import=True)

# apply the tagdoc function to each document
m.foreach(tagdoc)

# execute the processing steps defined in the steps variable of the 
# configuration object
m.process()

#
cfgs=(
    ('SVM',learning.learningSVM),
    ('KNN',learning.learningKNN),
#    ('NB', learning.learningNB),
)
for n,c in cfgs: 
    logout='/tmp/%s-%s-uni.res' % (prefix,n)
    print n, 'Unigram', logout
    m.learn(c,learning.datasetUnigramCFG,learn=True,copylog=logout)
for n,c in cfgs: 
    logout='/tmp/%s-%s-bi.res' % (prefix,n)
    print n, 'Bigram', logout
    m.learn(c,learning.datasetBigramCFG,learn=True,copylog=logout)

# execute the Learning Processing Resource to produce UNIGRAM features
# followed by the use of gate2arff.py to convert the GATE output to
# ARFF files (for different feature values: bin, tf, tfidf, tfidf2)
# Note: you can skip the producefeaturesCFG step if you did another
# learning step before, because the datafiles have then already been created.
featurevaluetypes=['-bin'] #,'-tf','-tfidf','-tfidf2']
m.learn(learning.learningProduceFeaturesCFG,learning.datasetUnigramCFG)
for t in featurevaluetypes:
    #gate2arff.convert("/tmp/savedFiles",
    #                  open("/tmp/%s-%s-uni.arff" % (prefix,t),'w'),gate2arff.gentypes[t])
    # calling gate2arff via system call is MUCH MUCH FASTER
    os.system('python ../basescripts/gate2arff.py %s /tmp/savedFiles /tmp/%s%s-uni.arff' %
              (t,prefix,t))

# the same for bigrams
m.learn(learning.learningProduceFeaturesCFG,learning.datasetBigramCFG)
for t in featurevaluetypes:
    #gate2arff.convert("/tmp/savedFiles",
    #                  open("/tmp/%s-%s-bi.arff" % (prefix,t),'w'),gate2arff.gentypes[t])
    # calling gate2arff via system call is MUCH MUCH FASTER
    os.system('python ../basescripts/gate2arff.py %s /tmp/savedFiles /tmp/%s%s-bi.arff' %
              (t,prefix,t))

# cleanup the gate objects, sync the corpus back to disk
m.cleanup()
